# -*- coding: utf-8 -*-
import xbmcgui, xbmcplugin
import os
import json
from sys import argv
try: from urllib import unquote
except ImportError: from urllib.parse import unquote
from apis import tmdb_api as TMDb
from modules.nav_utils import build_url, add_dir, setView, notification, translate_path, execute_builtin, show_text
from modules.utils import select_dialog, local_string as ls
from modules import settings
# from modules.utils import logger

dialog = xbmcgui.Dialog()
addon_dir = translate_path('special://home/addons/plugin.video.fen')

icon = os.path.join(addon_dir, "icon.png")
fanart = os.path.join(addon_dir, "fanart.png")

icon_directory = settings.get_theme()

class People():
	def __init__(self, actor_info=None):
		self.image_base = 'https://image.tmdb.org/t/p/%s%s'
		if actor_info:
			self.actor_id = actor_info[0]
			self.choose_id = True if self.actor_id == 'None' else False
			self.actor_name = actor_info[1]
			self.actor_image = actor_info[2]
			try: self.page_no = actor_info[3]
			except: self.page_no = 1
			try: self.count = actor_info[4]
			except: self.count = 0
			self.__handle__ = int(argv[1])

	def main(self):
		params = {'actor_id': self.actor_id, 'actor_name': self.actor_name, 'actor_image': self.actor_image, 'page_no': self.page_no}
		menus = [(ls(32028), 'people_search.media_results', True, 'movies'), 
				(ls(32029), 'people_search.media_results', True, 'tvshows'),
				('%s %s' % (ls(32028), ls(32796)), 'people_search.media_results', True,  'movies_directed'),
				(ls(32797), 'people_search.biography_results', False),
				(ls(32798), 'people_search.image_results', False),
				('%s %s' % (ls(33045), ls(32798)), 'people_search.tagged_image_results', False),
				('%ss' % ls(32491), 'people_search.video_results', True)]
		if settings.addon_installed('script.extendedinfo'):
			menus.append((ls(32610), 'people_search.extended_info', False))
		for menu_item in menus:
			try: params['media_type'] = menu_item[3]
			except: pass
			self._add_dir(menu_item[0], menu_item[1], params, isFolder=menu_item[2])
		xbmcplugin.setContent(self.__handle__, 'files')
		xbmcplugin.endOfDirectory(self.__handle__)
		setView('view.main')

	def search(self):
		if self.choose_id:
			self.query = self._get_query()
			if not self.query: return
			self.actor_id, self.actor_name, self.actor_image = self._get_actor_details()
			if not self.actor_id: return notification(ls(32760))
		self.main()

	def extended_info(self):
		execute_builtin('RunScript(script.extendedinfo,info=extendedactorinfo,id=%s)' % self.actor_id)

	def biography_results(self):
		from modules.utils import calculate_age
		def _make_biography():
			age = None
			name = bio_info.get('name')
			place_of_birth = bio_info.get('place_of_birth')
			biography = bio_info.get('biography')
			birthday = bio_info.get('birthday')
			deathday = bio_info.get('deathday')
			if deathday: age = calculate_age(birthday, '%Y-%m-%d', deathday)
			elif birthday: age = calculate_age(birthday, '%Y-%m-%d')
			text = '\n[COLOR dodgerblue]%s[/COLOR] %s' % (ls(32823), name)
			if place_of_birth: text += '\n\n[COLOR dodgerblue]%s[/COLOR] %s' % (ls(32824), place_of_birth)
			if birthday: text += '\n\n[COLOR dodgerblue]%s[/COLOR] %s' % (ls(32825), birthday)
			if deathday: text += '\n\n[COLOR dodgerblue]%s[/COLOR] %s, aged %s' % (ls(32826), deathday, age)
			elif age: text += '\n\n[COLOR dodgerblue]%s[/COLOR] %s' % (ls(32827), age)
			if biography: text += '\n\n[COLOR dodgerblue]%s[/COLOR]\n%s' % (ls(32828), biography)
			return text
		bio_info = TMDb.tmdb_people_biography(self.actor_id)
		if bio_info.get('biography', None) in ('', None):
			bio_info = TMDb.tmdb_people_biography(self.actor_id, 'en')
		if not bio_info: return notification(ls(32490))
		text = _make_biography()
		return show_text(bio_info.get('name'), text=text)

	def image_results(self):
		from indexers.images import Images
		params = {'mode': 'people_image_results', 'actor_name': self.actor_name, 'actor_id': self.actor_id, 'actor_image': self.actor_image,
					'page_no': self.page_no, 'rolling_count': self.count}
		Images().run(params)

	def tagged_image_results(self):
		from indexers.images import Images
		params = {'mode': 'people_tagged_image_results', 'actor_name': self.actor_name, 'actor_id': self.actor_id, 'actor_image': self.actor_image,
					'page_no': self.page_no, 'rolling_count': self.count}
		Images().run(params)

	def video_results(self):
		from apis.imdb_api import people_get_imdb_id, imdb_videos
		imdb_id = people_get_imdb_id(self.actor_name, self.actor_id)
		try: videos_list = imdb_videos(imdb_id)
		except: return
		for item in videos_list:
			title = item['title']
			poster = item['poster']
			url_params = {'mode': 'imdb_videos_choice', 'videos': json.dumps(item['videos']), 'thumb': poster, 'autoplay': 'true'}
			url = build_url(url_params)
			listitem = xbmcgui.ListItem(title)
			listitem.setArt({'icon': poster, 'poster': poster, 'thumb': poster, 'fanart': fanart})
			xbmcplugin.addDirectoryItem(self.__handle__, url, listitem, isFolder=False)
		xbmcplugin.setContent(self.__handle__, 'images')
		xbmcplugin.endOfDirectory(self.__handle__)
		setView('view.images', 'images')

	def media_results(self, media_type, page_no, letter):
		from indexers.movies import Movies
		from indexers.tvshows import TVShows
		from modules.nav_utils import paginate_list
		from modules.utils import sort_for_article
		def _add_misc_dir(url_params, list_name=ls(32799), info=ls(32800), iconImage='item_next.png'):
			listitem = xbmcgui.ListItem(list_name)
			listitem.setArt({'icon': os.path.join(icon_directory, iconImage), 'fanart': fanart})
			listitem.setInfo('video', {'title': list_name, 'plot': info})
			if url_params['mode'] == 'build_navigate_to_page': listitem.addContextMenuItems([(ls(32784),"RunPlugin(%s)" % build_url({'mode': 'toggle_jump_to'}))])
			xbmcplugin.addDirectoryItem(handle=self.__handle__, url=build_url(url_params), listitem=listitem, isFolder=True)
		try:
			builder = Movies if media_type in ('movies', 'movies_directed') else TVShows
			function = TMDb.tmdb_movies_actor_roles if media_type == 'movies' else TMDb.tmdb_tv_actor_roles if media_type == 'tvshows' else TMDb.tmdb_movies_crew
			content = 'movies' if media_type in ('movies', 'movies_directed') else 'tvshows'
			key = 'title' if media_type in ('movies', 'movies_directed') else 'name'
			result = function(self.actor_id)
			data = sort_for_article(result, key, settings.ignore_articles())
			original_list = [{'media_id': i['id'], 'title': i[key]} for i in data]
			paginated_list, total_pages = paginate_list(original_list, page_no, letter)
			media_list = [i['media_id'] for i in paginated_list]
			params = {'list': media_list, 'action': 'people_search_%s' % media_type}
			item_list = builder(params).worker()
			xbmcplugin.addDirectoryItems(self.__handle__, item_list, len(item_list))
			if total_pages > page_no: _add_misc_dir({'mode': 'people_search.media_results', 'media_type': media_type, 'actor_id': self.actor_id,'actor_name': self.actor_name,
													'actor_image': self.actor_image, 'new_page': str(page_no + 1), 'new_letter': letter})
			xbmcplugin.setContent(self.__handle__, content)
			xbmcplugin.endOfDirectory(self.__handle__)
			setView('view.%s' % content, content)
		except: notification(ls(32760), 3000)

	def _get_query(self):
		if self.actor_name in ('None', '', None): actor_name = dialog.input('Fen', type=xbmcgui.INPUT_ALPHANUM)
		else: actor_name = self.actor_name
		return unquote(actor_name)

	def _get_actor_details(self):
		from modules.history import add_to_search_history
		try: actors = TMDb.tmdb_people_info(self.query)
		except: actors = None
		if not actors: return None, None, None
		if len(actors) == 1:
			actors = actors[0]
			actor_id = actors['id']
			actor_name = actors['name']
			try: image_id = actors['profile_path']
			except: image_id = None
			if not image_id: actor_image = os.path.join(icon_directory, 'genre_family.png')
			else: actor_image = 'https://image.tmdb.org/t/p/h632/%s' % image_id
		else:
			def _builder():
				for item in actors:
					known_for_list = [i.get('title', 'NA') for i in item['known_for']]
					known_for_list = [i for i in known_for_list if not i == 'NA']
					image = 'https://image.tmdb.org/t/p/h632/%s' % item['profile_path'] if item['profile_path'] else os.path.join(icon_directory, 'genre_family.png')
					yield {'line1': item['name'], 'line2': ', '.join(known_for_list) if known_for_list else '', 'icon': image}
			list_items = list(_builder())
			kwargs = {'items': json.dumps(list_items), 'heading': 'Fen', 'enumerate': 'false', 'multi_choice': 'false', 'multi_line': 'false'}
			selection = select_dialog(actors, **kwargs)
			if selection == None: return None, None, None
			actor_id = int(selection['id'])
			actor_name = selection['name']
			actor_image = 'https://image.tmdb.org/t/p/h632/%s' % selection['profile_path'] if selection['profile_path'] else os.path.join(icon_directory, 'genre_family.png')
		add_to_search_history(actor_name, 'people_queries')
		return actor_id, actor_name, actor_image

	def extras_person_data(self, person_name):
		from caches.fen_cache import cache_object
		if person_name:
			self.actor_name = person_name
			self.actor_image = None
		try: self.actor_name = self.actor_name.decode('utf-8')
		except: pass
		tmdb_api = settings.tmdb_api_key()
		string = "%s_%s" % ('tmdb_movies_people_search_actor_data', self.actor_name)
		url = 'https://api.themoviedb.org/3/search/person?api_key=%s&language=en-US&query=%s' % (tmdb_api, self.actor_name)
		result = cache_object(TMDb.get_tmdb, string, url, 4)
		result = result['results']
		self.actor_id = [item['id'] for item in result][0]
		if not self.actor_image: self.actor_image = [self.image_base % ('original', item['profile_path']) for item in result][0]
		self.main()

	def _add_dir(self, list_name, mode, url_params, isFolder=True):
		url_params['mode'] = mode
		actor_id = url_params['actor_id']
		actor_name = url_params['actor_name']
		actor_image = url_params['actor_image']
		list_name = '[B]%s :[/B] %s' % (actor_name.upper(), list_name)
		info = url_params.get('info', '')
		url = build_url(url_params)
		listitem = xbmcgui.ListItem(list_name)
		listitem.setArt({'icon': actor_image, 'poster': actor_image, 'thumb': actor_image, 'fanart': fanart, 'banner': actor_image, 'landscape': actor_image})
		listitem.setInfo('video', {'title': list_name, 'plot': info})
		xbmcplugin.addDirectoryItem(handle=self.__handle__, url=url, listitem=listitem, isFolder=isFolder)

def popular_people():
	from windows import open_window
	selected = open_window(('windows.thumbviewer', 'ThumbPopularPeopleViewerXML'), 'thumbviewer.xml', settings.skin_location())
	if not selected: return
	execute_builtin(selected[0] % build_url(selected[1]))
